% Read analysed data in MATLAB format and construct ascii NCAR output
% 
%--------------------------------------------------------------------------
%     COPYRIGHT 1999-2000 EISCAT SCIENTIFIC ASSOCIATION.
%     PERMISSION TO USE, COPY, AND DISTRIBUTE, FOR NON-COMMERCIAL PURPOSES,
%     IS HEREBY GRANTED WITHOUT FEE, PROVIDED THAT THIS LICENSE INFORMATION
%     AND COPYRIGHT NOTICE APPEAR IN ALL COPIES.
%--------------------------------------------------------------------------
% works with GUISDAP v1.65 
%
% Note that transmitter power and system temperature are not available
% in the GUISDAP output record
%
% Corrected July 1999 to produce true NCAR ascii records
% Modified January 2000 to split input files into multiple output
%    files, one for each calendar day, and to split CP4B into
%    individual records for each pointing direction.  Also added
%    construction of the input filelist on the fly to avoid problems
%    where it is not up to date or missing (needs isdirectory.m and
%    getfilelist.m from Jussi in the path)

clear all

% set up absent data value
AbsentData                      = -32767;
  
% set up five NCAR single-valued parameters 
Azimuth				= 130;
Elevation			= 140;
Scattering_half_angle		= 190;
System_temperature		= 482;
Peak_transmitted_power		= 486;

% set up nine NCAR multi-valued parameters 
Altitude 			= 110;
Log10_Ne			= 520;
Ti				= 550;
TeTi				= 570;
Vi				= 580;
error_Log10_Ne			=-520;
error_Ti			=-550;
error_TeTi			=-570;
error_Vi			=-580;

% disp(['Please ensure that a correct filelist.dat exists before proceeding!']);
disp(['Available data directories in /data/analysis/']);
eval(['!ls /data/analysis | grep -v ps | grep -v NCAR | grep -v tar']);
data_path = input( 'Data directory (in quotes please): ');

FileNameLenght = size(data_path);
LastPartOfFileName = data_path(11:FileNameLenght(2));

NCAR_output_file = sprintf('/data/analysis/NCAR_%s', data_path);

data_path = sprintf('/data/analysis/%s/', data_path);

% generate a filelist for the input directory in memory to avoid cases
% where the filelist.dat is not up to date.  Needs getfilelist.m and
% isdirectory.m in the path; if these are missing, use the following
% two lines instead:
% load([data_path, 'filelist.dat']);
% n = size(filelist);

[filelist,errmsg]= getfilelist(data_path);
n = length(filelist);

disp(' ');
disp('Source directory:');
disp(data_path);
disp('Output file:');
disp(NCAR_output_file);
disp(' ');

for i=1:n;
  file	= int2str(filelist(i));

  s	= size(file);
  for ipos = s(2):7           % pre-pad filename to 8 characters
    file = ['0' file];   
  end

  disp(['processing ',file,'.mat (',num2str(i),'/',num2str(n(1)),')']);
  eval(['load ',data_path,file]);
  
  % output will be NCAR format data records (character version)

  KREC	=1101;				   % kind of record (data, character version)
  switch name_site
    case 'L',   KINST = 95;                % NCAR instrumant code: EISCAT Svalbard Radar
    case 'K',   KINST = 71;                %                       EISCAT Kiruna UHF
    case 'T',   KINST = 72;                %                       EISCAT Tromso UHF
    case 'S',   KINST = 73;                %                       EISCAT Sodankyla UHF
    case 'V',   KINST = 74;                %                       EISCAT Tromso VHF
    otherwise,  KINST = AbsentData;        %                       unknown
  end    
  if ( KINST == 95)
    KINDAT=6800 + 10*str2num(name_expr(length(name_expr))) + 1; % experiment mode, 6831 = GUP3, first code (ac)
  else
    KINDAT=6000 + 100*str2num(name_expr(length(name_expr)-1)) + 2; % experiment mode, 6302 = CP3, version 1
  end
  IBYR	=fix(r_time(1,1));		   % beginning year
  IBDT	=fix(r_time(1,2)*100+r_time(1,3)); % beginning date
  IBHM	=fix(r_time(1,4)*100+r_time(1,5)); % beginning hour & min
  IBCS	=fix(r_time(1,6)*100);		   % beginning centisecond
  IEYR	=fix(r_time(2,1));		   % ending year
  IEDT	=fix(r_time(2,2)*100+r_time(2,3)); % ending date
  IEHM	=fix(r_time(2,4)*100+r_time(2,5)); % ending hour & min
  IECS	=fix(r_time(2,6)*100);		   % ending centisecond
  LPROL	=16;				   % length of prologue
  JPAR	=5;				   % # single-valued parameters
  MPAR	=9;				   % # multi-valued parameters	
  NROW	=length(r_h);			   % # entries for each multi-valued parameter
  LTOT  =1+fix((JPAR+19)/20)*2+fix((MPAR+19)/20)*(NROW+1); % length of this record

  % Assemble output vector of codes and values for the multi-valued parameters
  
  Output_vector=[Altitude, ...
	Log10_Ne,Ti,TeTi,Vi, ...
	error_Log10_Ne,error_Ti,error_TeTi,error_Vi];
  for Ic = 1:NROW
    if ( r_status(Ic) == 0 )
      % only output results for fit parameter 0
      Output_vector = [Output_vector,round(r_h(Ic))];
      Output_vector = [Output_vector,round(log10(r_param(Ic,1))*1000)];
      Output_vector = [Output_vector,round(r_param(Ic,2))];
      Output_vector = [Output_vector,round(r_param(Ic,3)*1000)];
      % invert sign of GUISDAP velocities to match NCAR convention
      Output_vector = [Output_vector,round(-r_param(Ic,5))];
      Output_vector = [Output_vector,round(log10(r_error(Ic,1))*1000)];
      Output_vector = [Output_vector,round(r_error(Ic,2))];
      Output_vector = [Output_vector,round(r_error(Ic,3)*1000)];
      Output_vector = [Output_vector,round(r_error(Ic,5))];
    else
      % otherwise output altitude plus absent data values
      Output_vector = [Output_vector,round(r_h(Ic))];
      for Icc = 1:(MPAR-1)
	Output_vector = [Output_vector,AbsentData];
      end
    end
  end
 
  % replace infinities and all values > 32767, i.e. larger than a
  % signed short, by AbsentData value

  Ik = [find(Output_vector == Inf), ...
	find(Output_vector == -Inf), ...
	find(abs(Output_vector) > 32767) ];
  Output_vector(Ik) = AbsentData*ones(size(Ik));
 
  % Cast azimuth and elevation into range +-180, 0-90 degrees
   
  if ( r_el > 90.0 )
    m_el = 180.0 - r_el;
    m_az = r_az + 180.00;
  else
    m_el = r_el;
    m_az = r_az;   
  end
  m_az = mod((m_az + 360.0), 360.0);
  if ( m_az > 180.0 )
    m_az = m_az - 360.0;
  end
 
  % Open the ncar output file

  NCAR_output_file = sprintf('/data/analysis/NCAR_%d-%02d-%02d%s', ...
      r_time(1,1:3), LastPartOfFileName);
  [fid_NCAR_output_file, message] = fopen(NCAR_output_file,'a');

  if name_expr ~= 'CP4B'
    
    % Write the prologue

    fprintf(fid_NCAR_output_file, ...
	'%6i%6i%6i%6i%6i%6i%6i%6i%6i%6i%6i%6i%6i%6i%6i%6i\n', ...
	LTOT,KREC,KINST,KINDAT,IBYR,IBDT,IBHM,IBCS, ...
	IEYR,IEDT,IEHM,IECS,LPROL,JPAR,MPAR,NROW);
    
    % Write codes and values for the single-valued parameters
    
    fprintf(fid_NCAR_output_file, ...
	'%6i%6i%6i%6i%6i\n', ...
	Azimuth,Elevation,Scattering_half_angle,System_temperature, ...
	Peak_transmitted_power,round(m_az*100),round(m_el*100), ...
	AbsentData,AbsentData,AbsentData);
    
    % Write codes and values for the multi-valued parameters
    
    fprintf(fid_NCAR_output_file, ...
	'%6i%6i%6i%6i%6i%6i%6i%6i%6i\n', ...
	Output_vector);

  else
    
    % CP4 result vectors contain data for two different beams so
    % split these into two separate records with individual headers
    
    LTOT  =1+fix((JPAR+19)/20)*2+fix((MPAR+19)/20)*(NROW/2+1);
    NROW = NROW/2;
    
    % Write the prologue for the west beam 

    fprintf(fid_NCAR_output_file, ...
	'%6i%6i%6i%6i%6i%6i%6i%6i%6i%6i%6i%6i%6i%6i%6i%6i\n', ...
	LTOT,KREC,KINST,KINDAT,IBYR,IBDT,IBHM,IBCS, ...
	IEYR,IEDT,IEHM,IECS,LPROL,JPAR,MPAR,NROW);
    
    % Write codes and values for the single-valued parameters,
    % correcting for the west beam pointing direction
    m_az = 345.0;
    
    fprintf(fid_NCAR_output_file, ...
	'%6i%6i%6i%6i%6i\n', ...
	Azimuth,Elevation,Scattering_half_angle,System_temperature, ...
	Peak_transmitted_power,round(m_az*100),round(m_el*100), ...
	AbsentData,AbsentData,AbsentData);
    
    % Write codes and values for the multi-valued parameters
    
    fprintf(fid_NCAR_output_file, ...
	'%6i%6i%6i%6i%6i%6i%6i%6i%6i\n', ...
	Output_vector(1:fix(9+(length(Output_vector)-9)/2)));

    % Write the prologue for the north beam 

    fprintf(fid_NCAR_output_file, ...
	'%6i%6i%6i%6i%6i%6i%6i%6i%6i%6i%6i%6i%6i%6i%6i%6i\n', ...
	LTOT,KREC,KINST,KINDAT,IBYR,IBDT,IBHM,IBCS, ...
	IEYR,IEDT,IEHM,IECS,LPROL,JPAR,MPAR,NROW);
    
    % Write codes and values for the single-valued parameters
    % correcting for the west beam pointing direction
    m_az = 359.5;
    
    fprintf(fid_NCAR_output_file, ...
	'%6i%6i%6i%6i%6i\n', ...
	Azimuth,Elevation,Scattering_half_angle,System_temperature, ...
	Peak_transmitted_power,round(m_az*100),round(m_el*100), ...
	AbsentData,AbsentData,AbsentData);
    
    % Write codes and values for the multi-valued parameters
    
    fprintf(fid_NCAR_output_file, ...
	'%6i%6i%6i%6i%6i%6i%6i%6i%6i\n', ...
	Output_vector(1:9));
    fprintf(fid_NCAR_output_file, ...
	'%6i%6i%6i%6i%6i%6i%6i%6i%6i\n', ...
	Output_vector(fix(10+(length(Output_vector)-9)/2):fix(length(Output_vector))));
  end
  
  % close the file

  status = fclose(fid_NCAR_output_file);
end;

disp('Done.');
