% chk_par1: Interface betwen the user supplied and internal control parameters 
% GUISDAP v.1.60 96-05-27 Copyright Asko Huuskonen and Markku Lehtinen
% 
% chk_par1 is the main interface between the user supplied control parameters and
% the internal GUISDAP control parameters. There is mostly a one-to-one correspondence
% between the parameters e.g.
% User parameter   GUISDAP parameter
% analysis_altit    a_altit
% analysis_integr   a_integr
% display_figures   di_figures
% integ_deffile     a_integdeffile
% The user parameters are local to the workspace whereas the GUISDAP parameters are global
% If any of the user parameters is not specified, or empty, the corresponding GUISDAP
% parameter will get the values specified in the routine.
%
% See also: an_start globals chk_par2 load_filelist

% check that the various path names have directory separators at the end
len=length(data_path); 
if len>0, char=data_path(len);
if char~=':' & char~='/' & char~='\', data_path(len+1)='/';end; end

len=length(result_path); 
if len>0,  char=result_path(len);
if char~=':' & char~='/' & char~='\', result_path(len+1)='/';end; end

len=length(path_GUP); char=path_GUP(len);
if char~=':' & char~='/' & char~='\', path_GUP(len+1)='/';end

len=length(path_exps); char=path_exps(len);
if char~=':' & char~='/' & char~='\', path_exps(len+1)='/';end

len=length(path_tmp); char=path_tmp(len);
if char~=':' & char~='/' & char~='\', path_tmp(len+1)='/';end

% The first if-block tries to locate the data source and produces variables 
% necessary for integration:
a_simul=[]; % If this variable present, the data is produced from theory (So clear it)
% The first 'if' looks for the integration definition file for Nigel Wade's integration
% package. If this variable is present and the file exists, the data comes through that package.

if exist('integ_deffile') 
  if exist(canon(integ_deffile,0))
    % The first branch looks for the integration definition file for Nigel Wade's integration
    % package. If this variable is present and the file exists, the data comes through that package.
    a_integdeffile=canon(integ_deffile);
    a_rawdata=1; % This flags that NW's package is to be used
    fprintf('*************************************************************\n')
    fprintf(' An integration definition file found\n')
    fprintf(' Data will be integrated by Nigel Wade''s integration front end\n')
    fprintf('*************************************************************\n')
  else
    fprintf('*************************************************************\n')
    fprintf([' File ', integ_deffile,' not found\n'])
    fprintf(' Stopping execution\n')
    fprintf('*************************************************************\n')
    error(' ')
  end
elseif exist('analysis_simul')
  % The second branch is for simulated data to be calculated from the theory
  % The presence of the variable a_simul is the flag, which is used later. Contents
  % a_simul(1): integration time
  % a_simul(2): controls the start time in steps of 7200 s, i.e. 2 hours.
  % a_simul(3): The transmitter power
  % a_simul(4): The background temperature
  % a_simul(5:6): Antenna azimuth and elevation
  a_simul=[0 1 1.2e6 100 300 180 90 ];
  a_simul(1:length(analysis_simul))=analysis_simul;
  a_year=2222;
  a_start=7200*a_simul(2);
  a_end=7200*a_simul(2)+a_simul(1);
else
  % The final branch is the normal case. Call to load_filelist checks whether EISCAT  .dtst
  % files are present at the data_path (a_rawdata=1) or whether matlab files are there
%  [d_filelist,a_rawdata]=load_filelist;
   a_rawdata=0;
   d_filelist=getfilelist(data_path);
  a_start=tosecs(analysis_start);  % the programs uses internally only seconds, 
  a_end  =tosecs(analysis_end);    % counted from the beginning of year
  a_year=analysis_start(1);       
  a_ind=0; 
                                 
  if exist('analysis_integr'),
      a_integr=analysis_integr;  
  else
    % If analysis_integr not present, produce integration times from the dump times
    % Then no (further) integration is performed by GUISDAP
    if a_rawdata
      a_integr=2;  % This should make sure that all dumps are analyzed separately
    else % This is for Matlab files. Note that the files may contain integrated data
      a_integr=diff([a_start d_filelist(find(d_filelist>a_start & d_filelist<=a_end))']);
    end
    a_skip=zeros(size(a_integr));
  end
  if exist('analysis_skip'),
    a_skip=analysis_skip;
  else                             % if skips were not defined, assume
    a_skip=zeros(size(a_integr));  % them to be zero
  end

  if a_rawdata
    % If EISCAT .dtst files were found, the integration definition file is produced here
    % The code transfers the contents of the GUISDAP control parameters to the control
    % parameters of Nigel's analysis package. The standard header lines are read first:
    fid=fopen(canon([path_GUP,'matfiles/integdefs']),'r');
    aa=fread(fid,inf,'char');
    fclose(fid);
    a_integdeffile=canon([path_tmp,'integdef'],0);
    fid=fopen(a_integdeffile,'w');
    fwrite(fid,aa,'char');
    if name_site=='T'; fprintf(fid,['data-source tromso\n']);
    elseif name_site=='K'; fprintf(fid,['data-source kiruna\n']);
    elseif name_site=='S'; fprintf(fid,['data-source sodankyla\n']);
    elseif name_site=='V'; fprintf(fid,['data-source vhf\n']);
    end
    fprintf(fid,['input-names ', canon(data_path),'\n']);
    fprintf(fid,'start-time %.0f:%0.f:%.0f',analysis_start(4:6));
    fprintf(fid,' %0.f/%.0f/%0.f\n', analysis_start(1:3));
    fprintf(fid,'end-time %.0f:%0.f:%.0f',analysis_end(4:6));
    fprintf(fid,' %0.f/%.0f/%0.f\n', analysis_end(1:3));
    fprintf(fid,'cycle-time %0.f\n', sum(a_integr)+sum(a_skip));
    starts=[0,cumsum(a_integr+a_skip)];
    for i=1:length(a_integr)
      fprintf(fid,'scan-pos timed %0.f  %0.f\n',starts(i), a_integr(i));
    end
    fclose(fid);
  end
end

a_control=[100000, 0.01, 6 1];
% a_control(1)  No fit is tried, if the error of Ne is larger than (1) at the start
% a_control(2)  Fitting is stopped when step for all parameters is less than (2)
% a_control(3)  Maximum number of iterations
% a_control(4)  Variance calculation
%               = 1 when variance estimated from data
%               = 2 when variance estimated using ambiguity functions
if exist('analysis_control')
  ind=find(analysis_control>0);
  a_control(ind)=analysis_control(ind);
end
if any(a_simul) & a_control(4)==1
  a_control(4)=2;
end

a_Magic_const=1;
if exist('Magic_const')
  a_Magic_const=Magic_const;
end
  

di_figures=[0 0 0 0]; 
if exist('display_figures'), 
  di_figures(1:length(display_figures))=display_figures;
end
if exist('display_fit'),  % For compatibility with GUISDAP 1.44 and earlier    
  di_figures(3)=display_fit; 
end
if exist('display_results'), % For compatibility with GUISDAP 1.44 and earlier    
  di_figures(4)=display_results; 
end



