% analysismain_NW: Main program for the data analysis
%
% note: GUIZARD and GUISPERT are called from here
%
% Main program for the Guisdap data analysis. 
%
% The most important operations performed are:
%   routine name    action
%   chk_par1_NW     1. transforms the user supplied control parameters to internal parameters
%                   2. checks the data source (matlab files/EISCAT .dtst files)
%   init_graphics   opens a sufficient number of figure windows and defines there sizes etc.
%   integr_NW       calls the Nigel Wade integration package, when EISCAT .dtst files are used
%   integr_data     integrates data from Matlab files
%   decodeparblock  transfers the radar parameters (power, etc.) to internal GUISDAP parameters
%   load_initfile   loads the ambiguity functions, etc.
%   scale_lpgwom    scales the spectal amb. function with the correlator algorithm factors (lpg_ND)
%   frontend        correct data problems and initialize for new mainland system data
%   radar_eq        radar equation     
%   scale_data      scales the data with the correlator algorithm factors (lpg_ND) 
%   subr_backgr     background subtraction to the data  
%   get_apriori     the a priori model, electron density obtained from the data
%   half_prof       performs the gated analysis to the data 
%   save_results    results stored to the disk
%   plot_results    displays the results
%
% Other routines called:
%   globals
%   nat_const
%   get_ADDRSHIFT 
%   load_GUPvar
%   GUIZARD
%   GUISPERT
%   form_adpar
%   spektri_init
%   constants
%   chk_par2
%   simul_dump
%   clear_results
% 
% 199905       version developed from an_start by TG  placed under change control
% 2001         NW version NWan_start
% 20011002 bi  renamed start_analysis_NW (tired of a-tab and an-tab not finding analysis.m)
% 20011002 bi  added tau1v
% 20011102 bi  added tau1, tau1av, tau1avsingle, cp7hv
% 20011104 bi  renamed analysismain_NW to better reflect it's function
% 20011104 bi  added call to frontend, immediately before GUISPERT
%              commands inserted into GUISPERT will override settings from frontend

  t_init
  t_start(1)
  globals % defines (nearly) all global variables 
  global antennaEfficiency calTemp name_expr
  chk_par1_NW
  nat_const
  get_ADDRSHIFT
  init_graphics

  rcprog_old=-1;
  EOF=0;
  while ~EOF,
 
  if any(a_simul)
    OK=1;EOF=1;
  else
    if a_rawdata,
      [OK,EOF]=integr_NW;
    else
      [OK,EOF]=integr_data;
    end
    if OK, decodeparblock; end
  end

  %*************************************************************************************
  %
  % At this point an integrated complex data dump is stored in variable d_data.
  % It is possible to study the contents of the data by graphical and other means,
  % e.g. ind=1:length(d_data)-1; plot(ind-1,real(d_data(ind)));
  %
  %*************************************************************************************
  
  if OK,
    if d_rcprog~=rcprog_old  
      load_initfile   
      if a_control(4)>=2, load_GUPvar, end  
      if any(a_simul), simulparblock; end
      if exist('GUIZARD')==2, GUIZARD, end
      scale_lpgwom % scales the spectral ambiguity function with lpg_ND factors
      form_adpar   
      spektri_init % Loads in plasma dispersion function table
      constants    
      rcprog_old=d_rcprog;
    end

    frontend % formerly included in GUISPERT

    if exist('GUISPERT')==2, GUISPERT, end

    radar_eq % calculates the radar constant      

    chk_par2   
    
    if any(a_simul),
      simul_dump
    else
      scale_data   
    end
    
    %*************************************************************************************
    %
    % Now the data has been scaled by calibration, so that it appears in units of K
    % ind=0:length(d_data)-2; plot(ind,real(d_data(ind))); % this would show the data
    %
    %*************************************************************************************
    
    if strcmp(name_expr,'tau0') ...
     | strcmp(name_expr,'tau1v') ...
     | strcmp(name_expr,'tau1sv') ...
     | strcmp(name_expr,'tau1_singel_v')
      fprintf('analysismain_NW: %s, no background subtraction!\n',name_expr)   
    else
      subr_backgr
    end
    
    %*************************************************************************************
    %
    % At this point one finds background subtracted data in vector d_data.
    %
    %*************************************************************************************
    
    if di_figures(1)                % ==> figure 'ReDump'   
      if ishandle(di_figures(1))
	set(0,'CurrentFigure',di_figures(1));
	ind = find(real(d_data>-200 & d_data<2000)); 
	plot(ind-1,real(d_data(ind))), title(' Correlator dump, real part'); 
	xlabel('Address'); ylabel('Power [K]'); grid; drawnow
	clear ind
      else
	di_figures(1) = 0;
      end    
    end    
    
    if di_figures(2)                % ==> figure 'PowerProfile'   
      if ~ishandle(di_figures(2))
	di_figures(2) = 0;
      end    
    end   
    
    if di_figures(3)                % ==> figure 'FitResult'   
      if ~ishandle(di_figures(3))
	di_figures(3) = 0;
      end    
    end    
    
    get_apriori(any(a_simul))
    
    %*************************************************************************************
    %
    % The get_apriori call calculated the raw electron density profile. 
    % It is stored in variables
    %   pp_range   : range to power measurements
    %   pp_profile : Ne with a priori temperature ratio model
    %   pp_sigma   : Ne with Te=Ti
    %
    %*************************************************************************************
    
    clear_results 

    %**************************************** 				
    half_prof 
    %**************************************** 

    if strcmp(name_expr,'tau1') ...
     | strcmp(name_expr,'tau1v') ...
     | strcmp(name_expr,'tau1av') ...
     | strcmp(name_expr,'tau1avsingle') ...
     | strcmp(name_expr,'tau1_singel_v') ...
     | strcmp(name_expr,'cp4bv') ...
     | strcmp(name_expr,'cp7hv')
      fprintf('analysismain_NW: reversing velocities for %s\n',name_expr)
      r_param(:,5) = -r_param(:,5);   
    end

    save_results
    
    fprintf('analysismain_NW: writing NCAR results file...\n')
    NCAR_output;
    fprintf('analysismain_NW: writing NCAR velocity file...\n')
    NCAR_output_velocity;
    
    if di_figures(4)                % ==> figure 'Results'
      if ishandle(di_figures(4))
	fprintf('analysismain_NW: plotting results with quick_look...\n')
	set(0,'CurrentFigure',di_figures(4));
	clf;
	quick_look;
      else
	di_figures(4) = 0;
      end    
    end 
    
  end
end
