% integr_data: Integration program for EISCAT data stored as Matlab files
% GUISDAP v1.60   96-05-27 Copyright Asko Huuskonen, Markku Lehtinen
%
% Integration program for radar data in Matlab files. The files may either contain
% individual dumps or integrated dumps. The latter may or may not have 
% the variance estimates of the data included.
% Each call to this routine performs the integation for one integration period
%
% NOTE: This is a EISCAT specific function
%
% Input parameters (all global):
% a_ind a_indold a_interval a_year a_start a_integr a_skip a_end 
%     : specify the integration periods
% d_filelist : contains the names of the data files
% Output parameters:
% OK  : if true, the integration was succesful
% EOF : if true, the end of file was found during integration
% Output parameters (global):
% d_parbl : the parameter block returned by the integration program
% d_data : the integrated data vector
% d_var1 d_var2: data variances
%
% See also: an_start integr_NW integrate
% function  [OK,EOF]=integr_data
function  [OK,EOF]=integr_data
 
global d_parbl d_data d_var1 d_var2  data_path d_filelist a_control 
global a_ind a_indold a_interval a_year a_start a_integr a_skip a_end 
 
OK=0;EOF=0;
if a_ind==0
  a_ind=1;
  a_interval=a_start+[0 ,a_integr(1)];
else
   a_indold=a_ind;a_ind=a_ind+1; if (a_ind>length(a_integr)), a_ind=1;end
   a_interval=a_interval(2)+a_skip(a_indold)+[0 ,a_integr(a_ind)];
end
if a_interval(2)>=a_end; EOF=1; end
if (d_filelist(length(d_filelist))<a_interval(1)), EOF=1; return, end, 

fixed=[9:10];  % parameters which are not allowed to change
averaged=[8];  % parameters which are averaged
ORed=[];       % parameter which is OR'ed

for file = d_filelist(find(d_filelist>a_interval(1) & d_filelist<=a_interval(2)))
  filename='00000000';str=int2str(file);filename(9-length(str):8)=str;
  load(canon([data_path, filename]))
  d_parbl=col(d_parbl);

  [secs,year]=tosecs(d_parbl(1:6));
  if (~(secs>=file & (file-secs)<1 & year==a_year)),
    disp('Filename conflicts with file contents or years do not match'), keyboard
  end

  if isreal(d_data) % change to complex
    lendata=length(d_data);
    d_data(lendata-1:lendata)=abs(d_data(lendata-1:lendata));
    d_data=d_data(1:2:lendata)+sqrt(-1)*d_data(2:2:lendata);
		end
		
%  if d_parbl(95)~=0, fprintf(' Status word is %g\n',d_parbl(95)), end 
  dumpOK= 1;%rem(d_parbl(95),2)==0 & d_parbl(95)~=64;
  if ~OK & dumpOK,  % initialize with the first good dump
    first_parbl=d_parbl;         % save the first parameter block
    prev_parbl=d_parbl;          % initialize the previous parameter block
    aver=d_parbl(averaged);      % initialize averaging
    status=d_parbl(ORed);        % save the status word
    starttime=secs-d_parbl(7);  % calculate starttime of first dump
    if (exist('pre_integrated') == 1), 
      aver=d_parbl(averaged)*i_averaged;      % initialize averaging
      data=d_data;
      if exist('i_var1') % The integrated file need not have variances in it
        d_var1=i_var1(:);
        d_var2=i_var2(:);
        N_averaged=i_averaged;
      else
        d_var1=d_data.*d_data; 
        d_var2=d_data.*conj(d_data);
        N_averaged=1;
      end
    else
      aver=d_parbl(averaged);      % initialize averaging
      data=d_data;                 % data is now a complex column vector
      d_var1=d_data.*d_data;       % for the variance calculations
      d_var2=d_data.*conj(d_data);
      N_averaged=1;
    end
    OK=1;
  elseif OK & dumpOK,  % update with the following files
  % if any(d_parbl(fixed)~=first_parbl(fixed)),
  %   disp(' changes in the parameter block, continuing')
  %   indfixed=find(d_parbl(fixed)~=first_parbl(fixed));
  %   disp('           #  ,  original  , last dump')
  %   indfixed=fixed(indfixed);
  %   disp([indfixed',first_parbl(indfixed),d_parbl(indfixed)]) 
  % end
    if any(d_parbl(fixed)~=prev_parbl(fixed)),
   %  disp(' change in the parameter block')
      indfixed = find(d_parbl(fixed)~=prev_parbl(fixed));
      disp('    param#  previous   current ')
      indfixed = fixed(indfixed);
      disp([indfixed',prev_parbl(indfixed),d_parbl(indfixed)]) 
    end
    status=bitwiseor(status,d_parbl(ORed),16);
    if (exist('pre_integrated') == 1), 
      aver=aver+d_parbl(averaged)*i_averaged;
      data=data+d_data;
      if exist('i_var1')
        d_var1=d_var1+i_var1(:);
        d_var2=d_var2+i_var2(:);
        N_averaged=N_averaged+i_averaged;
      else
        d_var1=d_var1+d_data.*d_data;
        d_var2=d_var2+d_data.*conj(d_data);
        N_averaged=N_averaged+1;
      end
    else
      aver=aver+d_parbl(averaged);
      data=data+d_data;
      d_var1=d_var1+d_data.*d_data;
      d_var2=d_var2+d_data.*conj(d_data);
      N_averaged=N_averaged+1;
    end
  end
  prev_parbl=d_parbl; % update previous parameter block
end

if OK,  % if at least one good data dump was found
  if a_control(4)==1 & N_averaged<2,
    fprintf(' One file is not enough for variance determination\n')
    fprintf(' Skipping this integration period\n')
    fprintf(' command ''analysis_control(4)=2'' in the startup file will enable the analysis\n')
    OK=0;
    return
  elseif a_control(4)==1 & N_averaged<5,
    fprintf(' *******************    WARNING    **********************************\n')
    fprintf(' %.0f files may not be enough for reliable variance determination\n',N_averaged)
    fprintf(' *******************    WARNING    **********************************\n')
  end

  % update parameter block, accept the last parameter block as starting point
  d_parbl(averaged)=aver/N_averaged;     
  d_parbl(ORed)=status;
  d_parbl(7)=secs-starttime;
  d_data=data;
  d_var1=d_var1-data.*data/N_averaged;
  d_var2=d_var2-data.*conj(data)/N_averaged;

end
