function [y,mo,d,h,mi,s] = datevec(t)
%DATEVEC Date components.
%   C = DATEVEC(T) separates the components of date strings and date
%   numbers into date vectors containing [year month date hour mins
%   secs] as columns.  If T is a date string, it must be in one of the
%   date formats 0,1,2,6,13,14, 15,16 (as defined by DATESTR).
%
%   [Y,M,D,H,MI,S] = DATEVEC(T) returns the components of the date
%   vector as individual variables.
%
%   Examples
%     d = '12/24/1984';
%     t = 725000.00;
%     c = datevec(d) or c = datevec(t) produce c = [1984 12 24 0 0 0].
%     [y,m,d,h,mi,s] = datevec(d) returns y=1984, m=12, d=24, h=0, mi=0, s=0.
%
%   See also DATENUM, DATESTR, CLOCK.

%   Copyright (c) 1984-96 by The MathWorks, Inc.
%   $Revision: 1.10 $  $Date: 1996/09/30 14:58:57 $

if nargin < 1
  error('Not enough input arguments.');
end

pm = 0;
[date_row,date_col] = size(t);

if isstr(t)
  dts = zeros(date_row,6);
  siz = [date_row 1];
  for count = 1:date_row 
   % Convert date input to date vector
   % Initially, the six fields are all unknown.
   c(1,1:6) = [NaN NaN NaN NaN NaN NaN];
   d = [' ' lower(t(count,:)) ' '];
   
   % Replace 'a ', 'am', 'p ' or 'pm' with ': '.
   
   p = max(find(d == 'a' | d == 'p'));
   if ~isempty(p)
      if (d(p+1) == 'm' | d(p+1) == ' ') & d(p-1) ~= lower('e')
         pm = (d(p) == 'p');
         if d(p-1) == ' '
            d(p-1:p+1) = ':  ';
         else
            d(p:p+1) = ': ';
         end
      end
   end
   
  % Any remaining letters must be in the month field; interpret and delete them.
   p = find(isletter(d));
   if ~isempty(p)
      k = min(p);
      if d(k+3) == '.', d(k+3) = ' '; end
      M = ['jan'; 'feb'; 'mar'; 'apr'; 'may'; 'jun'; ...
           'jul'; 'aug'; 'sep'; 'oct'; 'nov'; 'dec'];
      c(2) = find(all((M == d(ones(12,1),k:k+2))'));
      d(p) = setstr(' '*ones(size(p)));
   end
   
   % Find all nonnumbers.
   
   p = find((d < '0' | d > '9') & (d ~= '.'));
   
   % Pick off and classify numeric fields, one by one.
   % Colons delinate hour, minutes and seconds.
   
   k = 1;
   while k < length(p)
      if d(p(k)) ~= ' ' & d(p(k)+1) == '-'
         f = str2num(d(p(k)+1:p(k+2)-1));
         k = k+1;
      else
         f = str2num(d(p(k)+1:p(k+1)-1));
      end
      if ~isempty(f)
         if d(p(k))==':' | d(p(k+1))==':'
            if isnan(c(4))
               c(4) = f;             % hour
               if pm
                  c(4) = f+12;
               end
            elseif isnan(c(5))
               c(5) = f;             % minutes
            elseif isnan(c(6)) 
               c(6) = f;             % seconds
            else
               error(['Too many time fields in ' t])
            end
         elseif isnan(c(2))
            if f > 12
               error([num2str(f) ' is too large to be a month.'])
            end
            c(2) = f;                % month
         elseif isnan(c(3))
            c(3) = f;                % date
         elseif isnan(c(1))
            if (f >= 0) & (p(k+1)-p(k) == 3)
               c(1) = f + 1900;      % year in 20th century
            else
               c(1) = f;             % year
            end
         else
            error(['Too many date fields in ' t])
         end
      end
      k = k+1;
   end

   if sum(isnan(c)) >= 5
      error(['Cannot parse date ' t])
   end

   % Check for unspecified year and set it to the current year
   if isnan(c(1)), clk = clock; c(1) = clk(1); end
   
   % If any field has not been specified, set it to zero. 
   p = find(isnan(c));
   if ~isempty(p)
      c(p) = zeros(1,length(p));
   end

   dts(count,:) = c;
  end
  c = dts;
else
  siz = size(t);
  t = t(:); % Make sure t is a column vector.
  len = length(t);
 
  % Convert scalar date to date vector.

  % If necessary, make t positive.
  % Shift by an integer multiple of 400 years, which is p days.
  p = 400*365+97;
  shift = max(0,ceil(-t ./ p));
  t = t + shift*p;

  % Now convert to seconds, then peel off components.

  t = 86400*t;                   % seconds.
  c(:,6) = rem(t,60);
  t = floor(t);                  % Truncate to the nearest second
  t = floor(t ./ 60);            % minutes.
  c(:,5) = rem(t,60);
  t = floor(t ./ 60);            % hours.
  c(:,4) = rem(t,24);
  t = floor(t ./ 24);            % days.
  a = 365+97/400;
  y = floor(t ./ a);             % year

  i = find(t <= 365*y + ceil(0.25*y)-ceil(0.01*y)+ceil(0.0025*y));
  if ~isempty(i)
    y(i) = y(i) - 1;
  end
  c(:,1) = y - shift*400;

  dpm = [31 28 31 30 31 30 31 31 30 31 30 31];
  dpm = dpm(ones(len,1),:);
  t = t - (365*y + ceil(0.25*y)-ceil(0.01*y)+ceil(0.0025*y));
  i = find((rem(y,4) == 0 & rem(y,100) ~= 0) |  rem(y,400) == 0);
  dpm(i,2) = 29+zeros(size(i));
   
  cdm = cumsum(dpm')';                     % dpm rows may now differ
  cdm = [zeros(len,1) cdm(:,1:11)];
  c(:,2) = sum((t(:,ones(1,12)) - cdm > 0)')';
  c(:,3) = t - cdm(len*(c(:,2)-1)+(1:len)');
  
end

% Make sure time part is properly rounded, the day number is within
% range, and the first five fields are integers.
maxc = ones(size(c,1),1)*[24 60 60];
[e,col] = find(any((c(:,4:6) >= maxc)')' | ...
               any((c(:,3) > eomday(c(:,1),c(:,2)))')' | ...
               any((c(:,1:5) ~= floor(c(:,1:5)))')');
if ~isempty(e),
  dn = datenum(c(e,1),c(e,2),c(e,3),c(e,4),c(e,5),c(e,6));
  t = datevec(dn);
  if dn < 1, % Time only
    c(e,4:6) = t(:,4:6);
  else
    c(e,:) = t;
  end
end

if nargout <= 1
  y = c;
else
  y = reshape(c(:,1),siz);
  mo = reshape(c(:,2),siz);
  d = reshape(c(:,3),siz);
  h = reshape(c(:,4),siz);
  mi = reshape(c(:,5),siz);
  s = reshape(c(:,6),siz);
end
